----------------------------------------------------------------------------------
-- Create Date:    10:37:32 06/06/2012 

----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use work.config.all;
------------------------------------------------------------------------------
------------------------------------------------------------------------------
-- For a number of 8 bits, the max value is 256, so we need 4 bits in the
-- units, 4 bits in the tens and 2 bits in the hundreds

--The principle is simple:
--1. Check the 4 bit numbers in the Hundreds, Tens, and Units. If they are greater or equal to 5, then add 3 to it.
--2. Shift the binary number left one bit.
--3. Repeat from the top (for a 8 bit binary number, do it 8 times)

-- For a number of 12 bits, num_bcd will be of 16 (4 * 4) bits. Thus, num will 
-- be of 28 (16 + 12) bits


----------------------------------------------------------------------------
----------------------------------------------------------------------------

entity con_binary_bcd is
    Port (
        clk     : in   STD_LOGIC;
        peak    : in   STD_LOGIC_VECTOR (RAMP_BITS-1 downto 0);
        num_bcd : out  STD_LOGIC_VECTOR (NUMBCD_BITS-1 downto 0));
end con_binary_bcd;

architecture Behavioral of con_binary_bcd is
constant SUMBCD_BITS : natural := RAMP_BITS + NUMBCD_BITS; 

begin

process(clk, peak)
variable number: std_logic_vector(SUMBCD_BITS-1 downto 0); 
begin
    if (rising_edge(clk)) then
        number := "0000000000000000" & peak;   -- NUMBCD_BITS bits of 0
		  
        for i in 0 to RAMP_BITS-1 loop    -- Do ramp_bits times 
        -- for the 4 units
            if number(SUMBCD_BITS-1 downto SUMBCD_BITS-4) >= "0101" then
                number(SUMBCD_BITS-1 downto SUMBCD_BITS-4) := std_logic_vector (unsigned(number(SUMBCD_BITS-1 downto SUMBCD_BITS-4)) + 3); 
            end if; 
            
            if number(SUMBCD_BITS-5 downto SUMBCD_BITS-8) >= "0101" then
                number(SUMBCD_BITS-5 downto SUMBCD_BITS-8) := std_logic_vector (unsigned(number(SUMBCD_BITS-5 downto SUMBCD_BITS-8)) + 3); 
            end if; 
            
            if number(SUMBCD_BITS-9 downto SUMBCD_BITS-12) >= "0101" then
                number(SUMBCD_BITS-9 downto SUMBCD_BITS-12) := std_logic_vector (unsigned(number(SUMBCD_BITS-9 downto SUMBCD_BITS-12)) + 3); 
            end if; 
            
            if number(SUMBCD_BITS-13 downto SUMBCD_BITS-16) >= "0101" then
                number(SUMBCD_BITS-13 downto SUMBCD_BITS-16) := std_logic_vector (unsigned(number(SUMBCD_BITS-13 downto SUMBCD_BITS-16)) + 3); 
            end if; 

            number(SUMBCD_BITS-1 downto 0) := number(SUMBCD_BITS-2 downto 0) & '0'; 
        end loop;
		  
        num_bcd <= number(SUMBCD_BITS-1 downto RAMP_BITS); 
    end if; 

end process; 

end Behavioral;

