library IEEE;
library work;

use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use work.config.all;

entity da2ctrl is
    Port ( 
       clk   : in  std_logic;  -- 100Mhz
       reset : in  std_logic;
     
       data1 : in  std_logic_vector(RAMP_BITS-1 downto 0);
       data2 : in  std_logic_vector(RAMP_BITS-1 downto 0);
       start : in  std_logic;
       done  : out std_logic;
     
       out1  : out std_logic;
       out2  : out std_logic;
       sync  : out std_logic;
       sclk  : out std_logic
    );
end da2ctrl;

architecture Behavioral of da2ctrl is
    constant conf        : std_logic_vector(3 downto 0) := "0000";
    type state_type is (idle_st, start_st, sync_st, shift_st, done_st);  -- done_st to gaurantee a stable output of DAC  
    signal state         : state_type;
    signal temp1         : std_logic_vector(DA2_BITS-1 downto 0);
    signal temp2         : std_logic_vector(DA2_BITS-1 downto 0);
    signal t : integer range 0 to DA2_BITS-1;
    signal d : integer range 0 to 30;
begin

process(clk, reset)
begin
    if rising_edge(clk) then
        if reset = '1' then
            state <= idle_st;
            sync  <= '0';
            done  <= '0';
            temp1 <= (others => '0');
            temp2 <= (others => '0');
        else
            done <= '0';
            if start = '1' and state = idle_st then
                state <= start_st;
            end if;

            if state /= idle_st then
                if d = DA2COUNT then
                    case state is
                        when idle_st =>
                
                        when start_st =>
                            state <= sync_st;
                            sync  <= '1';
                            temp1 <= conf & data1;
                            temp2 <= conf & data2;
                    
                        when sync_st =>
                            state <= shift_st;
                            t     <= 0;
                            sync  <= '0';
                            
                        when shift_st =>
                            sync  <= '0';
                            temp1 <= temp1(DA2_BITS-2 downto 0)& "0";
                            temp2 <= temp2(DA2_BITS-2 downto 0)& "0";
                        
                            if t = DA2_BITS-1 then
                                state <= done_st;
                                t <= 0;
                            else
                                t <= t + 1;
                            end if;
                            
                        when done_st =>
                            done <= '1'; 
                            state <= idle_st;                           
                    end case;
                end if;
            end if;
        end if;
    end if;
end process;

process(clk, d, reset)
begin
    if rising_edge(clk) then
        if reset = '1' then
            d <= 0;
        else
            if d /= DA2COUNT then
                d <= d + 1;
            else
                d <= 0;
            end if;
        end if;
    end if;
end process;

out1 <= temp1(DA2_BITS-1);
out2 <= temp2(DA2_BITS-1);
sclk <= '1' when d < DA2CLOCK else '0';
            
end Behavioral;