library ieee;
library work;

use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use work.config.all;

entity ramp_gen is
    port (
        clk    : in  std_logic;
        reset  : in  std_logic; 
        tick   : in  std_logic;
        sw     : in  std_logic; 
        val    : out std_logic_vector (RAMP_BITS-1 downto 0);
        init   : out std_logic; 
        mid    : out std_logic
    );
end ramp_gen;

architecture Behavioral of ramp_gen is
    constant MAX_VAL : natural := 2**RAMP_BITS-1;
    type state_type is (idle, rising, falling); 
    signal state_reg, state_next: state_type; 
    signal aux_reg, aux_next: integer range 0 to MAX_VAL; 
begin

-- FSMD state & date registers
process (clk, reset, sw)
begin
    if rising_edge(clk) then
        if sw = '1' or reset = '1' then
            state_reg <= idle;
            aux_reg   <= 0; 
        else
            state_reg <= state_next; 
            aux_reg   <= aux_next; 
        end if;
    end if; 
end process; 

-- Next state logic & data path 
process (state_reg, aux_reg, tick)
begin   
        state_next <= state_reg; 
        aux_next   <= aux_reg;        
    
        case state_reg is 
        
            when idle =>
                if tick = '1' then 
                    state_next <= rising; 
                end if; 
                
            when rising =>
                if tick = '1' then
                    if aux_reg = MAX_VAL then
                        state_next <= falling; 
                    else
                        aux_next   <= aux_reg + 1; 
                    end if; 
                end if; 
            
            when falling =>
                if tick = '1' then
                    if aux_reg = 0 then
                        state_next <= rising; 
                    else
                        aux_next   <= aux_reg - 1; 
                    end if; 
                end if; 
     end case;
end process; 

-- Output logic

val  <= std_logic_vector (to_unsigned(aux_reg, RAMP_BITS)); 
init <= '1' when (state_reg = rising  and aux_reg = 0       ) else '0';
mid  <= '1' when (state_reg = falling and aux_reg = MAX_VAL ) else '0';

end Behavioral;
