--------------------------------------------------------------------------------
-- Create Date:   15:15:24 05/11/2012
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
 
entity test_restoring is
end test_restoring;
 
architecture behavior of test_restoring is 
    --Inputs
    signal clk         : std_logic := '0';
    signal reset       : std_logic := '0';
    signal numerator   : unsigned(7 downto 0) := (others => '0');
    signal denumerator : unsigned(4 downto 0) := (others => '0');
    signal restart     : std_logic := '0';

     --Outputs
    signal quotient    : unsigned(3 downto 0);
    signal div_done    : std_logic;

    -- Clock period definitions
    constant clk_period : time := 10 ns;
begin
    uut: entity work.restoring(behavioral)
    generic map (
        num_bits    => 8,
        den_bits    => 5,
        quot_bits   => 4
    )
    port map (
        clk         => clk,
        reset       => reset,
        numerator   => numerator,
        denumerator => denumerator,
        restart     => restart,
        quotient    => quotient,
        div_done    => div_done
   );

   -- Clock process definitions
   clk_process :process
   begin
        clk <= '0';
        wait for clk_period/2;
        clk <= '1';
        wait for clk_period/2;
   end process;
 

   -- Stimulus process
   stim_proc: process
   begin        
      reset <= '1';
      wait for 100 ns;  
      reset <= '0';
      wait for clk_period*10;

      numerator   <= "11111111";
      denumerator <= "10000";
      restart     <= '1', '0' after clk_period;
      wait for clk_period * 30;
      
      numerator   <= "11111111";
      denumerator <= "11111";
      restart     <= '1', '0' after clk_period;
      wait for clk_period * 30;
      
      numerator   <= "10101111";
      denumerator <= "10101";
      restart     <= '1', '0' after clk_period;
      wait for clk_period * 30;
      
      wait;
   end process;

end;
